# Generated by Django 3.0.3 on 2020-05-04 23:44

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('auth', '0011_update_proxy_permissions'),
        ('permafrost', '0009_permafrostrole_group_update'),
    ]

    operations = [
        migrations.AlterField(
            model_name='permafrostrole',
            name='group',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='permafrost_role', to='auth.Group', verbose_name='Group'),
        ),
    ]
