# Generated by Django 3.1.3 on 2020-11-20 09:42

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('permafrost', '0016_merge_20200729_1716'),
    ]

    operations = [
        migrations.DeleteModel(
            name='PermafrostCategory',
        ),
    ]
