# Generated by Django 3.0.3 on 2020-03-10 22:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('permafrost', '0003_auto_20200227_2356'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='permafrostrole',
            options={'permissions': (('add_user_to_role', 'Can Add Users to Role'),), 'verbose_name': 'Permafrost Role', 'verbose_name_plural': 'Permafrost Roles'},
        ),
        migrations.AddField(
            model_name='permafrostrole',
            name='deleted',
            field=models.BooleanField(default=False, help_text='Soft Delete the Role', verbose_name='Deleted'),
        ),
    ]
