# Generated by Django 3.0.7 on 2020-06-05 23:15

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('permafrost', '0011_auto_20200506_2128'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='permafrostrole',
            name='category',
        ),
    ]
