# Generated by Django 3.0.7 on 2020-07-29 17:16

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('permafrost', '0012_auto_20200729_1710'),
        ('permafrost', '0015_auto_20200606_0042'),
    ]

    operations = [
    ]
